#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#include <math.h>

#define SCREEN_WIDTH 128
#define SCREEN_HEIGHT 64
#define OLED_RESET    -1
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define BLOCK_POSITION_X 100 // Pozycja X klocka
#define BLOCK_POSITION_Y 55  // Pozycja Y klocka
#define BLOCK_SIZE_X 6
#define BLOCK_SIZE_Y 9 // Rozmiar klocka
#define GRAVITY 0.1          // Siła grawitacji
#define MAX_WIND_STRENGTH 10 // Maksymalna siła wiatru
#define WINNING_SCORE 3      // Wynik potrzebny do wygranej
#define RESTART_BUTTON_PIN 9

int player1Score = 0;
int player2Score = 0;
int cannonAngle = 0;           // Początkowy kąt działa
bool angleIncreasing = true;   // Czy kąt wzrasta
bool ballFired = false;        // Czy kula została wystrzelona
float ballX = 0;               // Pozycja X kuli
float ballY = 0;               // Pozycja Y kuli
float ballSpeedX = 0;          // Prędkość X kuli
float ballSpeedY = 0;          // Prędkość Y kuli
float windSpeed = 0;           // Siła wiatru
int currentPlayer = 1;         // Aktualny gracz
int FIRE_BUTTON_PIN = 4;
// Dodajemy zmienną do śledzenia czasu trzymania przycisku
unsigned long buttonHoldStartTime = 0;
unsigned long chargeTime = 0;
const unsigned long maxChargeTime = 2000; // Maksymalny czas ładowania siły rzutu w milisekundach
float throwPower = 0;
void setup() {
  pinMode(4, INPUT_PULLUP);
  pinMode(10, INPUT_PULLUP);
  pinMode(RESTART_BUTTON_PIN, INPUT_PULLUP);
  display.setRotation(2);
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) {
    Serial.println(F("SSD1306 allocation failed"));
    for(;;);
  }
  display.display();
  delay(1000);
  display.clearDisplay();

  // Inicjalizacja generatora liczb losowych
  randomSeed(analogRead(0));
  windSpeed = random(-MAX_WIND_STRENGTH, MAX_WIND_STRENGTH)/10.0;
}

void loop() {
  display.clearDisplay();

  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0,0);
  display.print("Player 1: ");
  display.println(player1Score);

  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0,10);
  display.print("Player 2: ");
  display.println(player2Score);

  // Wyświetlanie aktualnego gracza
  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 20);
  display.print("Current Player: ");
  display.println(currentPlayer);

  // Rysowanie klocka
  display.fillRect(BLOCK_POSITION_X, BLOCK_POSITION_Y, BLOCK_SIZE_X, BLOCK_SIZE_Y, SSD1306_WHITE);

  // Rysowanie działa
  display.drawLine(0, SCREEN_HEIGHT, 10 * cos(cannonAngle * PI / 180), SCREEN_HEIGHT - 10 * sin(cannonAngle * PI / 180), SSD1306_WHITE);

  // Rysowanie siły wiatru
  drawWindIndicator(windSpeed);

  // Rysowanie kuli
  if (ballFired) {
    display.drawPixel(ballX, ballY, SSD1306_WHITE);
  }

  
  if(currentPlayer==1){
    FIRE_BUTTON_PIN=10;
  }
  else {
    FIRE_BUTTON_PIN=4;  
  }
  if (!ballFired && digitalRead(FIRE_BUTTON_PIN) == LOW) {
    if (buttonHoldStartTime == 0) {
      buttonHoldStartTime = millis(); // Rozpoczęcie mierzenia czasu trzymania przycisku
    }
    // Obliczanie czasu trzymania przycisku
    chargeTime = millis() - buttonHoldStartTime;
    // Ograniczenie czasu ładowania siły rzutu do maksymalnego czasu
    chargeTime = min(chargeTime, maxChargeTime);
    // Obliczanie siły rzutu na podstawie czasu ładowania
   throwPower = map(chargeTime, 0, maxChargeTime, 1, 6); // Mapowanie czasu trzymania na siłę rzutu
    // Wyświetlanie paska siły rzutu
    drawPowerBar(chargeTime * 1.0 / maxChargeTime);
    
  } else {
    // Jeśli przycisk został puszczony, wystrzel kulę z uwzględnieniem siły rzutu
    if (buttonHoldStartTime != 0) {
      ballFired = true;
      ballX = 0;
      ballY = SCREEN_HEIGHT;
      ballSpeedX = throwPower * cos(cannonAngle * PI / 180);
      ballSpeedY = -throwPower * sin(cannonAngle * PI / 180);
      buttonHoldStartTime = 0; // Resetowanie czasu trzymania przycisku
      chargeTime = 0;
    }
  }
  if (ballFired) {
    // Aktualizacja pozycji kuli pod wpływem siły grawitacji i siły wiatru
    ballX += ballSpeedX;
    ballY += ballSpeedY;
    ballSpeedY += GRAVITY;
    ballX += windSpeed;

    // Sprawdzenie kolizji z klockiem
    if (ballX >= BLOCK_POSITION_X && ballX <= BLOCK_POSITION_X + BLOCK_SIZE_X && ballY >= BLOCK_POSITION_Y && ballY <= BLOCK_POSITION_Y + BLOCK_SIZE_Y) {
          if(currentPlayer == 1)
          {
          player1Score++;
          currentPlayer = 2;
          windSpeed = random(-MAX_WIND_STRENGTH, MAX_WIND_STRENGTH)/10.0;
          }
         else {
          player2Score++;
          currentPlayer = 1;
          windSpeed = random(-MAX_WIND_STRENGTH, MAX_WIND_STRENGTH)/10.0;
        }
        ballFired = false;
    }

    // Sprawdzenie kolizji z ekranem
    if (ballY >= SCREEN_HEIGHT || ballX <= 0 || ballX >= SCREEN_WIDTH) {
      ballFired = false;
     windSpeed = random(-MAX_WIND_STRENGTH, MAX_WIND_STRENGTH)/10.0;
    // Zmiana aktualnego gracza
    currentPlayer = (currentPlayer == 1) ? 2 : 1;
    }
  }

  // Automatyczne zmienianie kąta działa
  if(buttonHoldStartTime == 0)
  {
  if (angleIncreasing) {
    cannonAngle=cannonAngle+3;
    if (cannonAngle >= 90) {
      angleIncreasing = false;
    }
  } else {
    cannonAngle=cannonAngle-3;
    if (cannonAngle <= 0) {
      angleIncreasing = true;
    }
  }
  }
   if (digitalRead(RESTART_BUTTON_PIN) == LOW){
  restartGame();
  }
  // Sprawdzenie warunku zakończenia gry
  if (player1Score >= WINNING_SCORE || player2Score >= WINNING_SCORE) {
    displayHappyAnimation();
    delay(2000); // Czas wyświetlania animacji
    restartGame(); // Rozpoczęcie nowej gry po zakończeniu animacji
  }
  display.display();
} //koniec pętli



// Funkcja do rysowania strzałki wiatru
void drawWindIndicator(float windSpeed) {
  float scaleFactor = 16; // Skalowanie strzałki
  int arrowSize = 3;        // Rozmiar strzałki
  int length = windSpeed * scaleFactor;
  // Obliczanie pozycji strzałki na podstawie wartości wiatru
  //int arrowPosition = 110 + (windSpeed * scaleFactor);
int arrowPosition = 110;


  // Rysowanie strzałki w zależności od kierunku wiatru
  if (windSpeed > 0) {
    // Wiatr w prawo
    display.drawLine(arrowPosition, 5, arrowPosition - arrowSize, 2, SSD1306_WHITE);
    display.drawLine(arrowPosition, 5, arrowPosition - arrowSize, 8, SSD1306_WHITE);
  } else if (windSpeed < 0) {
    // Wiatr w lewo
    display.drawLine(arrowPosition, 5, arrowPosition + arrowSize, 2, SSD1306_WHITE);
    display.drawLine(arrowPosition, 5, arrowPosition + arrowSize, 8, SSD1306_WHITE);
  }
  display.drawLine(arrowPosition, 5, arrowPosition - length, 5, SSD1306_WHITE);
  
// Rysowanie siły wiatru
  display.setTextSize(1);
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(arrowPosition-9,10);
  display.print(windSpeed,1);
} 

// Funkcja do rysowania paska siły rzutu
void drawPowerBar(float power) {
  int barWidth = 50;
  int barHeight = 4;
  int posX = (64 - barWidth) / 2;
  int posY = SCREEN_HEIGHT - barHeight - 20;
  int filledWidth = power * barWidth;
  
  // Rysowanie ramki paska
  display.drawRect(posX, posY, barWidth, barHeight, SSD1306_WHITE);
  // Wypełnienie paska w zależności od siły rzutu
  display.fillRect(posX, posY, filledWidth, barHeight, SSD1306_WHITE);
}
// Funkcja restartująca grę
void restartGame() {
  player1Score = 0;
  player2Score = 0;
  currentPlayer = 1;
  ballFired = false;
}

// Funkcja wyświetlająca animację zadowolonego ludka
void displayHappyAnimation() {
  for (int i = 0; i < 5; i++) {
      
      {
        for(int j=0; j<12;j++)
        {
        display.clearDisplay();
        display.setTextSize(1);
        display.setTextColor(SSD1306_WHITE);
        display.setCursor(15,5);
        display.print("Wygrywa ");
        if(player1Score == WINNING_SCORE)
          {display.print("Player1");}
        else {display.print("Player2");}

        display.drawCircle(64, 28, 5, SSD1306_WHITE);
        display.drawLine(64, 33, 64, 50, SSD1306_WHITE);
        display.drawLine(64, 50, 72, 64, SSD1306_WHITE);
        display.drawLine(64, 50, 56, 64, SSD1306_WHITE);
        display.drawLine(64, 40, 64-j, 45-j, SSD1306_WHITE);
        display.drawLine(64-j, 45-j, 64-j, 45-j-7, SSD1306_WHITE);
        display.drawLine(64, 40, 64+j, 45-j, SSD1306_WHITE);
        display.drawLine(64+j, 45-j, 64+j, 45-j-7, SSD1306_WHITE);
        
        display.display();
        delay(25);
        }
        for(int k=12; k>0;k--)
        {
        display.clearDisplay();
        display.drawCircle(64, 28, 5, SSD1306_WHITE);
        display.drawLine(64, 33, 64, 50, SSD1306_WHITE);
        display.drawLine(64, 50, 72, 64, SSD1306_WHITE);
        display.drawLine(64, 50, 56, 64, SSD1306_WHITE);
        display.drawLine(64, 40, 64-k, 45-k, SSD1306_WHITE);
        display.drawLine(64-k, 45-k, 64-k, 45-k-7, SSD1306_WHITE);
        display.drawLine(64, 40, 64+k, 45-k, SSD1306_WHITE);
        display.drawLine(64+k, 45-k, 64+k, 45-k-7, SSD1306_WHITE);
        display.setTextSize(1);
        display.setTextColor(SSD1306_WHITE);
        display.setCursor(15,5);
        display.print("Wygrywa ");
        if(player1Score == WINNING_SCORE)
          {display.print("Player1");}
        else {display.print("Player2");}
        display.display();
        delay(25);
        }
        
  }

  }
}
